/*
 * $Id$
 * Copyright (C) 2001 The Apache Software Foundation. All rights reserved.
 * For details on use and redistribution please refer to the
 * LICENSE file included with these sources.
 */

package org.apache.fop.pdf;

/**
 * class representing an action object.
 */
public abstract class PDFAction extends PDFObject {


    /**
     * create an Action object.
     * this constructor is used for passing on the object number to the PDFObject
     *
     * @param number the object's number
     */
    public PDFAction(int number) {

        /* generic creation of object */
        super(number);
    }

    /**
     * empty constructor for PDFAction.
     * this constructor is used when there is no additional object being created
     *
     */
    public PDFAction() {}

    /**
     * represent the action to call
     * this method should be implemented to return the action which gets
     * called by the Link Object.  This could be a reference to another object
     * or the specific destination of the link
     *
     * @return the action to place next to /A within a Link
     */
    abstract public String getAction();


    /**
     * represent the object in PDF
     * this method should be implemented to return the PDF which is to be
     * generated by the Action object
     *
     * @return the PDF string
     */
    abstract public byte[] toPDF();

}
